      PROGRAM RORA
C
C     VERSION 2.0  PAUL BARLOW, USGS FEBRUARY 15, 2012
C     THIS VERSION UPDATES THE 2007 VERSION PREPARED BY AL RUTLEDGE
C
C      ONLY CHANGE IS TO INCREASE MAXIMUM NUMBER OF YEARS FROM 120 
C       TO 150. LINES OF CODE THAT WERE CHANGED ARE PRECEDED BY A
C       V2.0 COMMENT LINE
C
C  THIS PROGRAM ESTIMATES GROUND-WATER RECHARGE FROM A DAILY-VALUES
C  RECORD OF STREAMFLOW, USING THE "RORABAUGH METHOD."  THIS AND
C  OTHER PROGRAMS ARE DOCUMENTED IN USGS WRIR 98-4148. THIS 2005
C  VERSION OF THE PROGRAM READS DAILY-VALUES FROM A FLAT-FILE.
C  (THE ORIGINAL VERSION READ FROM A "Z-FILE".)
C
C  THIS PROGRAM ALSO READS INFORMATION ABOUT GAGING STATIONS FROM
C  FILE "STATION.TXT" (DRAINAGE AREA).
C
C  SEVERAL DECLARATION STATEMENTS PERTAIN TO ARRAY SIZES:
C          MAXIMUM NUMBER OF YEARS = 150
C          MAXIMUM NUMBER OF DAYS = 44000.
C               (ALSO NOTE INITIALIZING STATEMENTS)
C          RETRIEVE DAILY VALUES AFTER THE YEAR 1890.
C          MAXIMUM NUMBER OF RECHARGE EVENTS (PEAKS) = 6000
C               (ALSO NOTE LINES BETWEEN LABELS 330 AND 340)
C V2.0
      COMMON/BIG/Q(150,12,31)
      CHARACTER*1 FLAG(150,12)
      REAL RECHMO(150,12)
C
      COMMON/BIG/Q1D(44000)
      COMMON/BIG/IYR1D(44000)
      COMMON/BIG/IMO1D(44000)
      COMMON/BIG/IDA1D(44000)
      COMMON/BIG/ALLGW(44000)
      COMMON/BIG/TA(6000)
      COMMON/BIG/TP(6000)
      COMMON/BIG/TBC(6000)
      COMMON/BIG/QP(6000)
      COMMON/BIG/QA(6000)
      COMMON/BIG/QB(6000)
      COMMON/BIG/QC(6000)
      COMMON/BIG/TS(6000)
      COMMON/BIG/TE(6000)
      COMMON/BIG/C(6000)
      COMMON/BIG/DELQ(6000)
      COMMON/BIG/RECH(6000)
      INTEGER TS, TE
      INTEGER OPT5
      REAL TA, TP, TBC, QP, QA, QB, QC, C, DELQ, RECH
      REAL Q
      REAL Q1D
      INTEGER IYR1D
      INTEGER IMO1D
      INTEGER IDA1D
      INTEGER ITBASE
      CHARACTER*1 ALLGW
      CHARACTER*16 INFILE, FNAME, FNM
      CHARACTER*80 YESNO
      REAL DA, K, RECHARG                                                      C
      CHARACTER*15 STANUM                                                      C
      CHARACTER*9 LINE
      REAL FLONUM
      CHARACTER*5 AGENCY                                                       C
      CHARACTER*180 BUF_LINE                                                   C
 
      IBEFORE=1890
 
      OPEN (UNIT=10,FILE='rorasum.txt')
      OPEN (UNIT=12,FILE='rorapek.txt')
      OPEN (UNIT=14,FILE='roramon.txt')
      OPEN (UNIT=15,FILE='roraqrt.txt')
      open (unit=16,file='roraWY.txt')
    1 READ (10,21,END=2)
      GO TO 1
    2 CONTINUE

C     NOTE: SOME COMPILERS MIGHT REQUIRE A LINE HERE "BACKSPACE 10"
C             THIS SHOULD NOT BE DONE IF RUNNING ON A DG.

      BACKSPACE 10

    5 FORMAT (2I6,3F11.2,1F8.2)
      WRITE (*,*) '  THIS PROGRAM DETERMINES RECHARGE BY '
      WRITE (*,*) 'THE "RORABAUGH METHOD." '

      WRITE (*,*) '    '
C
C------------------------- INITIALIZE VARIABLES : -------------------
C V2.0
      DO 10 IYEAR=1,150
      DO 10 IMONTH=1,12
      DO 10 IDAY=1,31
   10 Q(IYEAR,IMONTH,IDAY)=-999.0
      DO 11 I=1,44000
      ALLGW(I)= ' '
   11 Q1D(I)= -999.0
C
   13 FORMAT (F8.0)
   14 FORMAT (1I4,2I3,1I5, 1I12,1F11.1,1F7.1,2F8.3,1F7.3)
   15 FORMAT (A12,1F8.2)
   16 FORMAT (A12,1X,1F6.2,2X,1I4,'-',1I4,1I6,3X,1I8,3X,1F7.1,1F11.3)
   17 FORMAT (A12, 1F8.0)
   18 FORMAT (2F8.2, 2I8, 5F8.4,1I6)
   21 FORMAT (A6)
   23 FORMAT (1F12.0)
   25 FORMAT (I4)
   26 FORMAT (A4,1X,A8,1X,I4,1X,I2,1X,I2,1X,A)
   27 FORMAT (1I5, 2F10.1, 2I8)
C
C -------- READ THE DAILY-VALUES FILE OF STREAMFLOW: ------------------
C
      WRITE (*,*) 'GIVE THE NAME OF THE STREAMFLOW DAILY-VALUES  '             C
      WRITE (*,*) 'FILE  (the program is case-sensitive)         '             C
      WRITE (*,*) '(Example file that is included: "Indian.txt") '             C
      READ (*,'(A)') INFILE
      WRITE (*,*) 'READING FILE NAMED ',INFILE
      OPEN (UNIT=9, FILE=INFILE)
      IYEAR=1
   30 READ (9,21) LINE
      IF( LINE.NE.'agency') THEN
            GO TO 30
        END IF

      READ (9,21) LINE

      IFRSTYR=0


   35 READ (9,'(A)',END=37) BUF_LINE                                          C
      DO 36 J=1,180
      IF (BUF_LINE(J:J).EQ.achar(9)) BUF_LINE(J:J)= ','                       2007
   36 IF (BUF_LINE(J:J).EQ.'-') BUF_LINE(J:J)= ','                            2007
      READ (BUF_LINE,*,END=37) AGENCY,STANUM,IYEAR,IMONTH,IDAY,FLONUM         C


      IF(IFRSTYR.EQ.0) IFRSTYR=IYEAR                                           C
      IYEAR = IYEAR-IBEFORE                                                    C


      Q(IYEAR,IMONTH,IDAY) = FLONUM

      FLONUM = -999.0                                                         2007

      GO TO 35
   37 CONTINUE

      ILSTYR= IYEAR+IBEFORE
      WRITE (*,*) 'FIRST YEAR IN RECORD = ', IFRSTYR
      WRITE (*,*) ' LAST YEAR IN RECORD = ', ILSTYR



c
c --- flag nonexistent dates with flow=-9999 -----
c
      do 38 IYEAR=IFRSTYR, ILSTYR
      DO 38 IMONTH=1,12
      DO 38 IDAY=1,31
         IF((IMONTH.EQ.2).AND.(IDAY.GT.29)) THEN
            Q(IYEAR-IBEFORE,IMONTH,IDAY)= -9999.0
          END IF
         IF((IMONTH.EQ.2).AND.(IDAY.EQ.29)) THEN
           IDIV=INT((IYEAR)/4.0)
           XDIV=(IYEAR)/4.0
           DIFFER=ABS(IDIV-XDIV)
           IF(DIFFER.GT.0.1) THEN
               Q(IYEAR-IBEFORE,IMONTH,IDAY)= -9999.0
             END IF
          END IF
         IF(IDAY.EQ.31) THEN
              IF((IMONTH.EQ.4).OR.(IMONTH.EQ.6).OR.(IMONTH.EQ.9)
     $         .OR.(IMONTH.EQ.11)) THEN
                    Q(IYEAR-IBEFORE,IMONTH,IDAY)= -9999.0
               END IF
          END IF
   38 CONTINUE



      WRITE (*,*) '                 MONTH  '
      WRITE (*,*) ' YEAR   J F M A M J J A S O N D'

      IFRSTYR= IFRSTYR-IBEFORE
      ILSTYR= ILSTYR-IBEFORE

      DO 72 IYEAR=IFRSTYR, ILSTYR

           DO 55 IMONTH=1,12
   55      FLAG(IYEAR,IMONTH)='.'
           DO 70 IMONTH=1,12
             DO 60 IDAY=1,31
               IF(Q(IYEAR,IMONTH,IDAY).EQ.-999) FLAG(IYEAR,IMONTH)='X'
               IF(Q(IYEAR,IMONTH,IDAY).EQ.-99) FLAG(IYEAR,IMONTH)='X'
   60         CONTINUE
   70         CONTINUE
            WRITE (*,73) IYEAR+IBEFORE, (FLAG(IYEAR,IMONTH),IMONTH=1,12)


   72 CONTINUE

        WRITE (*,*) ' '
        WRITE (*,*) ' COMPLETE RECORD = .      INCOMPLETE = X  '
        WRITE (*,*) ' '


   73 FORMAT (1I6, 2X, 12A2)
      CLOSE (9,STATUS='KEEP')
C
C   ---------------   SELECT TIME PERIOD OF INTEREST:  ------------------
C
      WRITE (*,*) 'START IN WHICH YEAR? '
      READ (*,*) IYEARST
      WRITE (*,*) 'END IN WHICH YEAR? '
      READ (*,*) IYEAREN
      IIMAX=0
      DO 120 IIYR=IYEARST,IYEAREN
           IDIV=INT(IIYR/4.0)
           XDIV=IIYR/4.0
           DIFFER=ABS(IDIV-XDIV)
           IF(DIFFER.LT.0.1) THEN
               IIMAX=IIMAX+366
             ELSE
               IIMAX=IIMAX+365
            END IF
  120 CONTINUE
      IYEARST= IYEARST-IBEFORE
      IYEAREN= IYEAREN-IBEFORE
C
C ---- ASSIGN VALUES TO 1-DIMENSIONAL ARRAYS OF DISCHARGE AND DATE: ----
C
      ICOUNT= 0
      IBREAK= 0
      ITESTX= 0
      DO 180 IYEAR= IYEARST, IYEAREN
      DO 180 IMONTH= 1,12
      DO 180 IDAY= 1, 31
          SFLOW= Q(IYEAR,IMONTH,IDAY)

          IF(SFLOW.EQ.-9999) GO TO 180
          IF(SFLOW.EQ.-99.OR.SFLOW.EQ.-999) THEN
                 ITEST=0
             ELSE
                 ITEST=1
           ENDIF
          IF(ITEST.EQ.1.AND.ITESTX.EQ.0) THEN
                 IBREAK= IBREAK+1
                 IF(IBREAK.GT.1) THEN
                   WRITE (*,*) '***************************************'
                   WRITE (*,*) '*** THERE IS A BREAK IN THE STREAM- ***'
                   WRITE (*,*) '*** FLOW RECORD WITHIN THE PERIOD OF **'
                   WRITE (*,*) '*** INTEREST.  PROGRAM TERMINATION. ***'
                   WRITE (*,*) '***************************************'
                   GO TO 1000
                 ENDIF
           ENDIF
          ITESTX= ITEST
          IF(SFLOW.EQ.-99.OR.SFLOW.EQ.-999) GO TO 180

          ICOUNT= ICOUNT + 1
          Q1D(ICOUNT)= SFLOW
          IYR1D(ICOUNT)= IYEAR
          IMO1D(ICOUNT)= IMONTH
          IDA1D(ICOUNT)= IDAY
C               EST1D(ICOUNT)= EST(IYEAR,IMONTH,IDAY)                      c
  180 CONTINUE
      WRITE (*,*) 'NUMBER OF DAYS (WITH DATA) COUNTED =            ',
     $ ICOUNT
      WRITE (*,*) 'NUMBER OF DAYS THAT SHOULD BE IN THIS INTERVAL =',
     $ IIMAX
      WRITE (*,*) ' '
C
      IZERO=1
      DO 185 I=1,ICOUNT
      IF(Q1D(I).EQ.0.0) IZERO=0
  185 ALLGW(I)= ' '

      IF (IZERO.EQ.0) THEN

         WRITE (*,*) 'THE STREAMFLOW RECORD INCLUDES ZEROS.  THESE WILL'
         WRITE (*,*) 'BE SET EQUAL TO 0.01 CFS.  IS THIS OK?  (y or n) '
         WRITE (*,*) ' '
         READ (*,'(A)') YESNO
           IF (YESNO.EQ.'n') THEN
              WRITE (*,*) 'SPECIFIY THE FLOW OF YOUR CHOICE, FOR DAYS '
              WRITE (*,*) 'OF ZERO FLOW (IN CFS).  THIS MUST BE SOME  '
              WRITE (*,*) 'SMALL POSITIVE NUMBER.                     '
              READ (*,*) XNEWMIN
              DO 187 I=1,ICOUNT
                IF(Q1D(I).EQ.0.00) THEN
                   Q1D(I)= XNEWMIN
                 END IF
  187         CONTINUE
             ELSE
                 DO 188 I=1,ICOUNT
                   IF(Q1D(I).EQ.0.00) THEN
                      Q1D(I)= 0.01
                    END IF
  188            CONTINUE

             END IF


       END IF



C
C ------------ READ LIST FILE GIVING STATION PROPERTIES:  -------------
C
      WRITE (*,*) 'READING FILE NAMED station.txt'
      OPEN (UNIT=11,FILE='station.txt')
      DO 190 I=1,10
  190 READ (11,*)
  200 READ(11,17) FNAME, DA
      IF(FNAME.NE.INFILE) GO TO 200
      WRITE (*,*) 'FILE NAME:', FNAME
      WRITE (*,*) 'DRAINAGE AREA:', DA
      WRITE (*,*) ' '
      CLOSE (11, STATUS='KEEP')
      IF (DA.LT.1.0) THEN
         WRITE (*,*) '*** DRAINAGE AREA IS SMALLER THAN RECOMMENDED ***'
         WRITE (*,*) '**********  USE RESULTS WITH CAUTION ************'
         WRITE (*,*) ' '
       ENDIF
      IF (DA.GT.500.0) THEN
         WRITE (*,*) '*** DRAINAGE AREA IS LARGER THAN RECOMMENDED ***'
         WRITE (*,*) '*********  USE RESULTS WITH CAUTION ************'
         WRITE (*,*) ' '
       ENDIF

C
C  ---  DETERMINE THE MINIMUM NUMBER OF DAYS OF ANTECEDENT RECESSION  --
C  ---  TO INDICATE THAT STREAMFLOW IS TO BE CONSIDERED GROUND-WATER  --
C  ---  DISCHARGE.  OBTAIN FROM THE EQUATION DA**0.2, ROUNDED UPWARD  --
C
      ITBASE=0
  210 ITBASE= ITBASE + 1
      IF(ITBASE.GT.10) THEN
           WRITE (*,*) 'PROBLEMS WITH CALCULATION OF THE REQUIREMENT '
           WRITE (*,*) 'OF ANTECEDENT RECESSION. PROGRAM ENDED'
           GO TO 1000
       END IF
      IF (ITBASE.GT.DA**0.2) THEN
           GO TO 220
        ELSE
           GO TO 210
       END IF
  220 CONTINUE

C
C ------ ENTER RECESSION INDEX OR OBTAIN FROM FILE index.txt:   --------
C
      WRITE (*,*) 'DO YOU WANT TO OBTAIN THE RECESSION INDEX '
      WRITE (*,*) 'FROM FILE index.txt?  ONLY ANSWER "y" IF '
      WRITE (*,*) 'THERE IS A LINE IN "index.txt" THAT GIVES'
      write (*,*) 'THE RECESSION INDEX FOR THIS STATION.'

      WRITE (*,*) '           (ENTER y OR n)                    '
      READ (*,'(A)') YESNO
      IF (YESNO.EQ.'n') THEN
          WRITE (*,*) 'ENTER RECESSION INDEX (DAYS/LOGQ):'
          READ (*,*) K
          GO TO 250
       END IF
      WRITE (*,*) 'READING FILE index.txt FOR RECESSION'
      WRITE (*,*) 'INDEX.' 
      OPEN (UNIT=13, FILE='index.txt')
      DO 230 IREAD=1,10
  230 READ (13,21) LINE
  240 READ(13,15) FNM, K
      IF(FNM.NE.INFILE) GO TO 240

      WRITE (*,*) 'RECESSION INDEX TO BE USED (DAYS/LOG Q) = ', K
      CLOSE (13, STATUS='KEEP')
  250 CONTINUE


C
C ----- SPECIFY THE MAXIMUM ALLOWABLE NUMBER OF DAYS THAT CAN BE USED
C ----- AFTER A PEAK, TO DETERMINE THE GROUND-WATER DISCHARGE AFTER
C ----- THE PEAK: 
C
      IRECMAX= INT(0.2144*K)

C ---- ALLOW THE USER TO OVERRIDE THE DEFAULT METHOD OF EXECUTION.  -----
C ---- NOTE: THE ALLOWANCE FOR NON-DEFAULT EXECUTION, IS DIFFERENT  -----
C ---- FROM THE NON-DEFAULT EXECUTION IN PREVIOUS VERSIONS OF RORA ------

      OPT5 = 0

      WRITE (*,*) ' '

      WRITE (*,*) 'DO YOU WANT THE PROGRAM TO EXECUTE USING THE      '
      WRITE (*,*) 'DEFAULT VALUE FOR THE REQUIREMENT OF ANTECEDENT   '
      WRITE (*,*) 'RECESSION?                                        '
      WRITE (*,*) '               (y or n)   '
      READ (*,'(A)') YESNO
      IF(YESNO.EQ.'n') THEN

c  
      WRITE (*,*) ' '

      WRITE (*,*) ' '
      WRITE (*,*) 'BY DEFAULT, THE REQUIREMENT OF ANTECEDENT RECESSION'
      WRITE (*,*) 'IS THE FOLLOWING, IN DAYS ............... ', ITBASE
      WRITE (*,*) ' '
      WRITE (*,*) 'NOTE: AN INCREASE IN THIS VARIABLE CAN REDUCE THE '
      WRITE (*,*) 'INFLUENCE OF ERRORS DUE TO DIRECT SURFACE RUNOFF  '
      WRITE (*,*) 'BUT MAY ALSO INDUCE OTHER ERRORS BECAUSE THERE WILL'
      WRITE (*,*) 'BE A REDUCTION IN THE NUMBER OF PEAKS THAT ARE     '
      WRITE (*,*) 'DETECTED. '
      WRITE (*,*) ' '

         OPT5=1

         WRITE (*,*) 'ENTER THE NUMBER OF DAYS BY WHICH THE        '
         WRITE (*,*) 'REQUIREMENT OF ANTECEDENT RECESSION IS       '
         WRITE (*,*) 'TO BE INCREASED, OR SPECIFY ZERO IF YOU      '
         WRITE (*,*) 'WANT IT TO REMAIN UNCHANGED:                 '
         WRITE (*,*) '             (ENTER 1, 2, 3, OR 0)           '
  255    READ (*,*) IDIFF

         IF (IDIFF.LT.0.OR.IDIFF.GT.3) THEN
            WRITE (*,*) 'SPECIFY CHANGING THE REQUIREMENT OF ANTECEDENT'
            WRITE (*,*) 'RECESSION BY 1, 2, OR 3 OR ELSE ENTER 0.      '
            GO TO 255
           ELSE
            ITBASE= ITBASE+IDIFF
          ENDIF

         IF (IDIFF.EQ.0) OPT5= 0


      END IF

          
      IF(IRECMAX.LT.ITBASE) THEN
          WRITE (*,*) ' **********************************************'
          WRITE (*,*) ' THE REQUIREMENT OF ANTECEDENT RECESSION EXCEEDS'
          WRITE (*,*) ' THE TIME LIMIT FOR THE END OF RECESSION.  WILL '
          WRITE (*,*) ' SET TIME LIMIT TO BE EQUAL TO THE REQUIREMENT '
          WRITE (*,*) ' OF ANTECEDENT RECESSION.  RESULTS MAY BE      '
          WRITE (*,*) ' QUESTIONABLE.                                 '
          WRITE (*,*) ' **********************************************'
          IRECMAX= ITBASE
        END IF

C
C-----------------ON DATES PRECEEDED BY A RECESSION PERIOD,-------------
C-------------------------SET VARIABLE ALLGW='*'  ----------------------
C
      WRITE (*,*) ' FINDING DAYS OF SUFFICIENT ANTECEDENT RECESSION'
      DO 270 I= ITBASE+1, ICOUNT
         INDICAT=1
         IBACK= 0
  260    IBACK= IBACK + 1
            IF(Q1D(I-IBACK).LT.Q1D(I-IBACK+1)) INDICAT=0
            IF(IBACK.LT.ITBASE) GO TO 260
            IF(INDICAT.EQ.1) THEN
                   ALLGW(I)= '*'
               ELSE
                   ALLGW(I)= ' '
             END IF
  270 CONTINUE
C
C
C -- THE FOLLOWING LINES UP TO LABEL 800 DETERMINE THE LOCATION (IN TIME) --
C --------------------  OF PEAKS AND RECESSION PERIODS:  -------------------
C
C
C ------------------- LOCATE END OF FIRST RECESSION: ----------------
C
      WRITE (*,*) ' LOCATING FIRST RECESSION PERIOD'
      I= 0
  280 I= I+1
      IF(I.GT.ICOUNT) THEN
          WRITE (*,*) 'NO RECESSIONS'
          GO TO 1000
       END IF
      IF (ALLGW(I).NE.'*') GO TO 280
  290 I=I+1
      IF (I.GT.ICOUNT) THEN
         WRITE (*,*) 'CANNOT FIND END OF THE FIRST RECESSION '
         GO TO 1000
        END IF
      IF (ALLGW(I).NE.' ') GO TO 290
      I=I-1
      TA(1)= I
      QA(1)= Q1D(I)
      IPEAK= 0
C
C -------------    FIND STREAMFLOW AND DAY OF THE PEAK:     -----------
C
      WRITE (*,*) ' LOCATING PEAKS + REMAINING RECESSION PERIODS '
  330 CONTINUE
      IPEAK= IPEAK+1
      IF(IPEAK.GT.6000) THEN
           WRITE (*,*) 'TOO MANY PEAKS '
           GO TO 1000
        END IF
      QP(IPEAK)= Q1D(I)
      TP(IPEAK)= I
      ILOOK= I + 1
  340 CONTINUE
      IF(ILOOK.GE.ICOUNT) GO TO 800
      IF(ALLGW(ILOOK).EQ.'*') GO TO 350
         IF (Q1D(ILOOK).GE.QP(IPEAK)) THEN
              QP(IPEAK)= Q1D(ILOOK)
              TP(IPEAK)= ILOOK
           END IF
         IF(ALLGW(ILOOK).EQ.' ') THEN
              ILOOK= ILOOK+1
              GO TO 340
           END IF
  350 CONTINUE
        TBC(IPEAK)= TP(IPEAK) + (0.2144*K)

C
C  -----   FIND FIRST AND LAST DAYS OF RECESSION FOLLOWING THE PEAK: ----
C
      I= ILOOK
      TS(IPEAK)= I
  370 CONTINUE
      I= I+1
      IF(I.GT.ICOUNT) GO TO 380
      IF(ALLGW(I).NE.' ') THEN
          GO TO 370
        END IF
  380 CONTINUE
      I= I-1
      TE(IPEAK)= I
      IF (TE(IPEAK)-TP(IPEAK).GT.IRECMAX) THEN
           TE(IPEAK)= TP(IPEAK) + IRECMAX
         END IF
      IF (TE(IPEAK).LT.TS(IPEAK)) THEN
           TE(IPEAK)= TS(IPEAK)
         END IF

      I=I+1
      IF(I.LT.ICOUNT) GO TO 330

C
C ----  GO HERE WHEN ALL RECESSION PERIODS AND PEAKS HAVE BEEN LOCATED: ----
C
  800 CONTINUE

      NPEAKS= IPEAK - 1


      WRITE (*,*) 'NUMBER OF PEAKS=', NPEAKS

      RECHARG=0.0
C
C --- EXTRAPOLATE STREAMFLOW, DETERMINE RECESSION CURVE DISPLACEMENT, AND --
C ----------------- CALCULATE RECHARGE FOR FIRST PEAK:  --------------------
C
      QB(1)= QA(1)*10**(-1*(TBC(1)-TA(1))/K)
      SUM= 0.0
      DO 810 I= TS(1), TE(1)
         DQ= Q1D(I) - (QA(1)*10**(-1.0*(I-TA(1))/K))
  810    SUM= SUM + (DQ*SQRT(I-TP(1)))
      NRECS= TE(1)-TS(1)+1
      C(1)= SUM/NRECS
      DELQ(1)= C(1)/(SQRT(TBC(1)-TP(1)))
      RECH(1)= 2.0*0.0371901*DELQ(1)*K / (2.3025851*DA)     
      RECHARG= RECHARG + RECH(1)
      QC(1)= QB(1) + DELQ(1)
C
C -- EXTRAPOLATE STREAMFLOW, DETERMINE RECESSION CURVE DISPLACEMENT, AND --
C --------------- CALCULATE RECHARGE FOR ALL OTHER PEAKS: -----------------
C
      DO 840 IPEAK=2, NPEAKS
         QA(IPEAK)= QC(IPEAK-1)
         TA(IPEAK)= TBC(IPEAK-1)
         QB(IPEAK)= QA(IPEAK)*10**(-1*(TBC(IPEAK)-TA(IPEAK))/K)


         SUM= 0.0
         DO 820 I= TS(IPEAK), TE(IPEAK)
             IF(I.GT.TA(IPEAK)) THEN
                BASELIN= QA(IPEAK)*10**(-1*(I-TA(IPEAK))/K)
               ELSE
                BASELIN= C(IPEAK-1)/(SQRT(I-TP(IPEAK-1))) +
     $              QA(IPEAK-1)*10**(-1.0*(I-TA(IPEAK-1))/K)
              END IF
             DQ= Q1D(I) - BASELIN
  820    SUM= SUM + (DQ*SQRT(I-TP(IPEAK)))
         NRECS= TE(IPEAK) - TS(IPEAK) + 1
         C(IPEAK)= SUM/NRECS
         DELQ(IPEAK)= C(IPEAK) / (SQRT(TBC(IPEAK)-TP(IPEAK)))
         RECH(IPEAK)= 2.0*0.0371901*DELQ(IPEAK)*K / (2.3025851*DA)
         RECHARG= RECHARG + RECH(IPEAK)
         QC(IPEAK)= QB(IPEAK) + DELQ(IPEAK)
  840 CONTINUE
C
C
C --  WRITE CALCULATIONS FOR EACH RECHARGE EVENT, TO FILE "rorapek.txt"  --
C
      WRITE (12,*) ' THIS IS FILE rorapek.txt: PEAK BY PEAK CALCULATIONS 
     $ OF PROGRAM rora.'
      WRITE (12,*) ' '
      WRITE (12,*) ' NOTE: RESULTS AT THIS SMALL TIME SCALE ARE FOR SCRE
     $ENING PURPOSES '
      WRITE (12,*) ' AND SHOULD NOT BE REPORTED OR USED QUANTITATIVELY.
     $ USE FILES     '
      WRITE (12,*) ' OUTRORA.SUM OR OUTRORA.QRT INSTEAD. '
      WRITE (12,*) ' '
      WRITE (12,*) 'INPUT FILE = ', INFILE
      WRITE (12,*) 'TIME PERIOD = ', IYEARST+IBEFORE, IYEAREN+IBEFORE
      WRITE (12,*) 'K =', K
      WRITE (12,*) 'CRITICAL TIME = ', 0.2144*K
      WRITE (12,*) 'REQMT OF ANT. RECESSION : ', ITBASE
      WRITE (12,*) '..TP...TS...TE.....TA.....TBC.....QP....QA.....QB...
     $.QC.......C.....DELQ....RECH..YEAR..MON'
      DO 850 IPEAK=1, NPEAKS
      WRITE (12,855)  INT(TP(IPEAK)), TS(IPEAK), TE(IPEAK), TA(IPEAK),
     $ TBC(IPEAK), QP(IPEAK), QA(IPEAK), QB(IPEAK), QC(IPEAK),
     $  C(IPEAK), DELQ(IPEAK), RECH(IPEAK), 
     $  IBEFORE+IYR1D(INT(TP(IPEAK))), IMO1D(INT(TP(IPEAK)))
  850 CONTINUE
  853 FORMAT (A10, 2X, 1I2)
  855 FORMAT (3I5, 2F8.2, 1F7.1, 3F7.3, 1F8.3, 2F7.3, 1I6, 1I4)
C
C
C  ---  LONG-TERM RESULTS ARE DETERMINED AND WRITTEN TO FILE "RORASUM.TXT" ---
C
      WRITE (*,*) ' '

      IF (IIMAX.EQ.ICOUNT) THEN

          WRITE (10,16)  FNAME, DA,
     $      IYEARST+IBEFORE, IYEAREN+IBEFORE, ITBASE, NPEAKS, K,
     $      RECHARG/(IYEAREN-IYEARST+1)
          WRITE (*,*) ' '
          WRITE (*,*) 'SUMMARY RESULTS WRITTEN TO FILE "rorasum.txt" '

        ELSE
          WRITE (*,*) ' '
          WRITE (*,*) 'SUMMARY RESULTS ARE NOT WRITTEN TO FILE '
          WRITE (*,*) 'OUTRORA.SUM BECAUSE RECORD IS NOT COMPLETE'
          WRITE (*,*) 'FOR THE TIME PERIOD OF INTEREST. '
          WRITE (10,858) FNAME, DA,
     $                IYEARST+IBEFORE, IYEAREN+IBEFORE,
     $            ' *** record is not complete *** '
       ENDIF
  858 FORMAT ( A12,1X,1F6.2,2X,1I4,'-',1I4, A32)

C
C  ---MONTHLY RESULTS ARE DETERMINED AND WRITTEN TO FILE "RORAMON.TXT" ---
C
C V2.0
      DO 900 I=1, 150
      DO 900 J=1, 12
  900 RECHMO(I,J)= 0.0

      DO 920 IPEAK=1, NPEAKS
      RECHMO(IYR1D(INT(TP(IPEAK))),IMO1D(INT(TP(IPEAK))) ) =
     $  RECHMO(IYR1D(INT(TP(IPEAK))),IMO1D(INT(TP(IPEAK))) ) +
     $  RECH(IPEAK)
  920 CONTINUE
C V2.0
      DO 930 IYEAR=1, 150
      DO 930 IMONTH=1,12
         IF (FLAG(IYEAR,IMONTH).EQ.'X') RECHMO(IYEAR,IMONTH)= -99.99
  930 CONTINUE

      WRITE (14,*) '  '
      WRITE (14,*) 'STREAMFLOW FILE =                  ', INFILE
      WRITE (14,*) 'RECESSION INDEX (DAYS/LOG CYCLE) = ', K
      WRITE (14,*) 'REQUIREMENT OF ANT. RECESSION  =   ', ITBASE
      WRITE (14,*) 'PROGRAM VERSION = 2.0, FEBRUARY 2012 '               2012
      WRITE (14,*) '  '
      WRITE (14,*) '                        MONTHLY RECHARGE IN INCHES:'
      WRITE (14,*) '         J     F     M     A     M     J     J     A
     $     S     O     N     D   YEAR'

      WRITE (15,*) '  '
      WRITE (15,*) 'STREAMFLOW FILE =                  ', INFILE
      WRITE (15,*) 'RECESSION INDEX (DAYS/LOG CYCLE) = ', K
      WRITE (15,*) 'REQUIREMENT OF ANTEC. RECESSION =  ', ITBASE
      WRITE (15,*) 'PROGRAM VERSION = 2.0, FEBRUARY 2012 '                2012
      WRITE (15,*) '  '
      WRITE (15,*) '       QUARTER-YEAR RECHARGE IN INCHES:          '
      WRITE (15,*) '       --------------------------------          '
      WRITE (15,*) '         JAN-    APR-    JULY-   OCT-    YEAR    '
      WRITE (15,*) '         MAR     JUNE    SEPT    DEC     TOTAL   '
      write (16,*) ' '
      write (16,*) 'Results on the basis of the '
      write (16,*) 'water year (Oct 1 to Sept 30) '
      write (16,*) ' '
      write (16,*) '      Year                Total '
      write (16,*) '-------------------       ----- '

      TOTXX=0.0
      DO 940 I=IYEARST, IYEAREN
      SUMYR=0.0
      IMISS=0
      DO 932 J=1,12
      SUMYR= SUMYR + RECHMO(I,J)
        IF (RECHMO(I,J).LT.-90.0) THEN
           IMISS=1
          ELSE
           TOTXX= TOTXX + RECHMO(I,J)
         END IF
  932 CONTINUE
      IF(IMISS.EQ.1) SUMYR= -99.99
      WRITE (14,943) I+IBEFORE, (RECHMO(I,J),J=1,12), SUMYR

              AA = RECHMO(I,1)+RECHMO(I,2)+RECHMO(I,3)
                   IF (RECHMO(I,1).LT.-99) AA= -99.99
                   IF (RECHMO(I,2).LT.-99) AA= -99.99
                   IF (RECHMO(I,3).LT.-99) AA= -99.99
              BB = RECHMO(I,4)+RECHMO(I,5)+RECHMO(I,6)
                   IF (RECHMO(I,4).LT.-99) BB= -99.99
                   IF (RECHMO(I,5).LT.-99) BB= -99.99
                   IF (RECHMO(I,6).LT.-99) BB= -99.99
              CC = RECHMO(I,7)+RECHMO(I,8)+RECHMO(I,9)
                   IF (RECHMO(I,7).LT.-99) CC= -99.99
                   IF (RECHMO(I,8).LT.-99) CC= -99.99
                   IF (RECHMO(I,9).LT.-99) CC= -99.99

                if (I.gt.IYEARST) then
                   WY=DD+AA+BB+CC
                   IF(DD.LT.-99) WY= -99.99
                   IF(AA.LT.-99) WY= -99.99
                   IF(BB.LT.-99) WY= -99.99
                   IF(CC.LT.-99) WY= -99.99
                   WRITE (16,945) I+IBEFORE-1,I+IBEFORE,WY
                 END IF

              DD = RECHMO(I,10)+RECHMO(I,11)+RECHMO(I,12)
                   IF (RECHMO(I,10).LT.-99) DD= -99.99
                   IF (RECHMO(I,11).LT.-99) DD= -99.99
                   IF (RECHMO(I,12).LT.-99) DD= -99.99

              WRITE (15,944) I+IBEFORE, AA, BB, CC, DD, SUMYR

  940 CONTINUE
            WRITE (14,*) ' '
            WRITE (14,*) '                TOTAL OF MONTHLY AMOUNTS  = ',
     $                                                             TOTXX

  943 FORMAT (1I6, 13F6.2)
  944 FORMAT (1I6, 5F8.2)
  945 format ('Oct ',1i4,' to Sept ',1i4,3x,5f8.2)

      WRITE (14,*) ' '
      WRITE (14,*) ' RESULTS ON THE MONTHLY TIME SCALE SHOULD BE USED WI
     $TH CAUTION. '
      WRITE (14,*) ' FILES RORAQRT.TXT AND RORASUM.TXT GIVE RESULTS AT T
     $HE CORRECT  '
      WRITE (14,*) ' TIME SCALES (QUARTER YEAR, YEAR, OR MORE).  '
      WRITE (*,*) ' '
C     WRITE (*,*) 'THE RATIO OF MEAN RECHARGE FROM MONTHLY AMOUNTS TO '
C     WRITE (*,*) 'THE MEAN RECHARGE CALCULATED FOR THE TOTAL PERIOD OF'
C     WRITE (*,*) 'INTEREST = ', TOTXX/RECHARG
      WRITE (*,*) ' '
      WRITE (*,*) 'MONTHLY RESULTS ARE WRITTEN TO FILE "roramon.txt" '
      WRITE (*,*) '...AND QUARTER-YEAR RESULTS TO FILE "roraqrt.txt" '

 1000 CONTINUE
      CLOSE (10,STATUS='KEEP')
      CLOSE (12,STATUS='KEEP')
      CLOSE (14,STATUS='KEEP')
      CLOSE (15,STATUS='KEEP')
      CLOSE (16,STATUS='KEEP')

      WRITE (*,*) ' '
      WRITE (*,*) ' Click the "enter" key to terminate. '
      READ (*,'(A)') YESNO

      STOP
      END
